@@ -50,11 +50,11 @@ from trove_fm.tests.models import UserAgent, UsualSuspect
 
 # Apply migrations at beginning and end of testing session
 @pytest.fixture(scope="session")
-def apply_migrations(save_state):
+def apply_migrations(save_state=False):
     print(f"\nsave_state: {save_state}\n")
     warnings.filterwarnings("ignore", category=DeprecationWarning)
     os.environ["TESTING"] = "1"
-    config = Config("trove_fm/alembic.ini")
+    config = Config(os.environ["ALEMBIC_CONFIG"])
     alembic.command.upgrade(config, "head")
     yield
     if save_state is not True:
@@ -128,6 +128,10 @@ def create_authorized_client(app: FastAPI, client: AsyncClient) -> Callable:
             "password": person.password
         }
 
+        url_path = app.url_path_for("person:login-email-and-password")
+        print(f"\n\nURL PATH: {url_path}")
+        print(f"\n\nHeaders: {client.headers}")
+
         res = await client.post(
             app.url_path_for("person:login-email-and-password"),
             headers=request_headers,
@@ -135,6 +139,7 @@ def create_authorized_client(app: FastAPI, client: AsyncClient) -> Callable:
         )
         assert res.status_code == status.HTTP_200_OK
 
+        print(f"\n\nUSER AGENT: {user_agent.user_agent}")
         print(f"\n\nUSER AGENT STORES COOKIES: {user_agent.stores_cookies}")
 
         if user_agent.stores_cookies:
