@@ -1,3 +1,4 @@
+
 """
 TroveFM is an online store and headless CMS.
 
@@ -25,7 +26,7 @@ from typing import Callable, Optional, Type, Union
 from urllib.parse import urlparse
 
 from databases import Database
-from dirty_equals import IsDatetime, IsNow, IsPositiveInt, IsStr, IsTrueLike
+from dirty_equals import IsDatetime, IsNow, IsPositiveInt, IsStr
 from fastapi import FastAPI, HTTPException, status
 from httpx import AsyncClient
 from itsdangerous.exc import BadTimeSignature, SignatureExpired
@@ -38,7 +39,7 @@ from trove_fm.app.config import (
     ACCESS_TOKEN_EXPIRE_MINUTES, BASE_URL, JWT_ALGORITHM, JWT_AUDIENCE, MAIL_LINK_EXPIRES, SECRET_KEY
 )
 from trove_fm.app.db.repositories.person import PersonRepository
-from trove_fm.app.models.person import PersonBase, PersonUnverified
+from trove_fm.app.models.person import PersonPublic, PersonUnverified
 from trove_fm.app.services import auth_service, verification_service
 from trove_fm.app.services import security
 from trove_fm.tests.models import UsualSuspect
@@ -93,7 +94,6 @@ class TestPersonRegistration:
 
         person_unverified = res.json()
         assert person_unverified == {
-            "active": IsTrueLike(),
             "username": "kgodel@live.com",
             "email_label": "home",
             "name_first": "Kurt",
@@ -104,7 +104,7 @@ class TestPersonRegistration:
         }
 
         # ensure that the person now exists in the db
-        person_in_db = await person_repo.get_person_by_email(email=new_person_creds["username"], populate=False)
+        person_in_db = await person_repo.get_person_by_email(email=new_person_creds["username"], get_profile=False)
         assert person_in_db is not None
         assert person_in_db.username == new_person_creds["username"]
 
@@ -116,13 +116,15 @@ class TestPersonRegistration:
         })
         assert created_person == person_in_db.dict(exclude={
             "id",
+            "active",
             "app_role",
             "password",
             "salt",
             "email_login",
             "verified",
             "created_at",
-            "updated_at"
+            "updated_at",
+            "profile"
         })
 
     @pytest.mark.parametrize(
@@ -196,12 +198,12 @@ class TestPersonRegistration:
         "name_first, name_last, username, password, status_code",
         [
             pytest.param(
-                "", "Lebowski",
+                None, "Lebowski",
                 "onechillindude5@gmail.com", "w9sp3nbBTU!NnvV*KfaBZ#L.y",
                 201, id="last_name_provided"
             ),
             pytest.param(
-                "Dude", "",
+                "Dude", None,
                 "one.chillindude@live.com", "w9sp3nbBTU!NnvV*KfaBZ#L.y",
                 201, id="first_name_provided"
             ),
@@ -223,13 +225,21 @@ class TestPersonRegistration:
         When a user registers, they MUST provide AT LEAST a first name OR a last name.
         """
         person_repo = PersonRepository(db)
-        new_person_creds = {
-            "name_first": name_first,
-            "name_last": name_last,
-            "username": username,
-            "email_label": "home",
-            "password": password
-        }
+
+        if name_first:
+            new_person_creds = {
+                "name_first": name_first,
+                "username": username,
+                "email_label": "home",
+                "password": password
+            }
+        elif name_last:
+            new_person_creds = {
+                "name_last": name_last,
+                "username": username,
+                "email_label": "home",
+                "password": password
+            }
 
         # make sure person doesn't exist yet
         person_in_db = await person_repo.get_person_by_email(email=new_person_creds["username"])
@@ -242,16 +252,25 @@ class TestPersonRegistration:
         assert res.status_code == status_code
 
         person_unverified = res.json()
-        assert person_unverified == {
-            "active": IsTrueLike(),
-            "username": username,
-            "email_label": "home",
-            "name_first": name_first,
-            "name_last": name_last,
-            "link_expiration": IsNow(delta=MAIL_LINK_EXPIRES, iso_string=True, enforce_tz=False),
-            "message": f"Your confirmation email has been sent to {username}",
-            "help_message": f"Didn't get the link? Click here to resend: {BASE_URL}/person/verify/resend/"
-        }
+
+        if name_first:
+            assert person_unverified == {
+                "username": username,
+                "email_label": "home",
+                "name_first": name_first,
+                "link_expiration": IsNow(delta=MAIL_LINK_EXPIRES, iso_string=True, enforce_tz=False),
+                "message": f"Your confirmation email has been sent to {username}",
+                "help_message": f"Didn't get the link? Click here to resend: {BASE_URL}/person/verify/resend/"
+            }
+        elif name_last:
+            assert person_unverified == {
+                "username": username,
+                "email_label": "home",
+                "name_last": name_last,
+                "link_expiration": IsNow(delta=MAIL_LINK_EXPIRES, iso_string=True, enforce_tz=False),
+                "message": f"Your confirmation email has been sent to {username}",
+                "help_message": f"Didn't get the link? Click here to resend: {BASE_URL}/person/verify/resend/"
+            }
 
     @pytest.mark.asyncio
     async def test_person_cannot_register_without_name(
@@ -265,8 +284,6 @@ class TestPersonRegistration:
         """
         person_repo = PersonRepository(db)
         new_person_creds = {
-            "name_first": "",
-            "name_last": "",
             "username": "one.chillinduderino@outlook.com",
             "email_label": "home",
             "password": "w9sp3nbBTU!NnvV*KfaBZ#L.y"
@@ -276,7 +293,7 @@ class TestPersonRegistration:
         person_in_db = await person_repo.get_person_by_email(email=new_person_creds["username"])
         assert person_in_db is None
 
-        # send post request to create person and ensure it is successful
+        # send post request to create person
         res = await client.post(
             app.url_path_for("person:register-person-credentials"), json={"new_person_creds": new_person_creds}
         )
@@ -297,6 +314,100 @@ class TestPersonRegistration:
             ]
         }
 
+    @pytest.mark.parametrize(
+        "name_prefix, name_first, name_last, name_suffix, "
+        "username, email_label, password, "
+        "status_code, field, min_length",
+        [
+            pytest.param(
+                "", "Sh'ron", "Guest", "V",
+                "shron@trove.fm", "home", "ic8pazN_BD!N9z*a2i7LpARC2",
+                422, "name_prefix", 1, id="name_prefix"
+            ),
+            pytest.param(
+                "Mr.", "", "Guest", "V",
+                "shron@trove.fm", "home", "ic8pazN_BD!N9z*a2i7LpARC2",
+                422, "name_first", 1, id="name_first"
+            ),
+            pytest.param(
+                "Mr.", "Sh'ron", "", "V",
+                "shron@trove.fm", "home", "ic8pazN_BD!N9z*a2i7LpARC2",
+                422, "name_last", 1, id="name_last"
+            ),
+            pytest.param(
+                "Mr.", "Sh'ron", "Guest", "",
+                "shron@trove.fm", "home", "ic8pazN_BD!N9z*a2i7LpARC2",
+                422, "name_suffix", 1, id="name_suffix"
+            ),
+            pytest.param(
+                "Mr.", "Sh'ron", "Guest", "V",
+                "shron@trove.fm", "", "ic8pazN_BD!N9z*a2i7LpARC2",
+                422, "email_label", 1, id="email_label"
+            ),
+            pytest.param(
+                "Mr.", "Sh'ron", "Guest", "V",
+                "shron@trove.fm", "home", "",
+                422, "password", 16, id="password"
+            ),
+        ],
+    )
+    @pytest.mark.asyncio
+    async def test_included_fields_have_value(
+        self,
+        app: FastAPI,
+        client: AsyncClient,
+        db: Database,
+        name_prefix: str,
+        name_first: str,
+        name_last: str,
+        name_suffix: str,
+        username: str,
+        email_label: str,
+        password: str,
+        status_code: int,
+        field: str,
+        min_length: int
+    ) -> None:
+        """
+        When a user registers, they MUST a value for each field that is specified.
+
+        We DO NOT test for length of username, as its validation already requires it to be
+        a valid email address.
+        """
+
+        new_person_creds = {
+            "name_prefix": name_prefix,
+            "name_first": name_first,
+            "name_last": name_last,
+            "name_suffix": name_suffix,
+            "username": username,
+            "email_label": email_label,
+            "password": password
+        }
+
+        res = await client.post(
+            app.url_path_for("person:register-person-credentials"), json={"new_person_creds": new_person_creds}
+        )
+        assert res.status_code == status.HTTP_422_UNPROCESSABLE_ENTITY
+
+        person_unverified = res.json()
+        assert person_unverified == {
+            "detail": [
+                {
+                    "ctx": {
+                        "limit_value": min_length
+                    },
+                    "loc": [
+                        "body",
+                        "new_person_creds",
+                        field
+                    ],
+                    "msg": f"ensure this value has at least {min_length} characters",
+                    "type": "value_error.any_str.min_length"
+                }
+            ]
+        }
+
     @pytest.mark.asyncio
     async def test_person_saved_password_is_hashed_and_has_salt(
         self,
@@ -321,7 +432,7 @@ class TestPersonRegistration:
 
         # ensure that the person password is hashed in the db
         # and that we can verify it using our auth service
-        person_in_db = await person_repo.get_person_by_email(email=new_person_creds["username"], populate=False)
+        person_in_db = await person_repo.get_person_by_email(email=new_person_creds["username"], get_profile=False)
         assert person_in_db is not None
         assert person_in_db.salt is not None and person_in_db.salt != "123"
         assert person_in_db.password != new_person_creds["password"]
@@ -457,7 +568,6 @@ class TestVerifyEmail:
 
         person_unverified = res.json()
         assert person_unverified == {
-            "active": IsTrueLike(),
             "username": username,
             "email_label": email_label,
             "name_first": name_first,
@@ -475,7 +585,6 @@ class TestVerifyEmail:
 
         person_unverified_still = res.json()
         assert person_unverified_still == {
-            "active": IsTrueLike(),
             "username": username,
             "email_label": email_label,
             "name_first": name_first,
@@ -527,7 +636,6 @@ class TestVerifyEmail:
 
         person_unverified = res.json()
         assert person_unverified == {
-            "active": IsTrueLike(),
             "username": username,
             "email_label": email_label,
             "name_first": name_first,
@@ -544,14 +652,13 @@ class TestVerifyEmail:
         res = await client.get(verification_url, follow_redirects=True)
 
         assert res.status_code == status.HTTP_200_OK
-
+        print(res.json())
         assert res.json() == {
-            "active": IsTrueLike,
             "username": username,
             "email_label": email_label,
             "name_first": name_first,
             "name_last": name_last,
-            "id": IsPositiveInt
+            "id": IsPositiveInt()
         }
 
         # request to resend verification email
@@ -579,7 +686,7 @@ class TestAuthTokens:
     async def test_can_create_access_token_successfully(
         self, app: FastAPI, client: AsyncClient, person_muir: UsualSuspect
     ) -> None:
-        person = PersonBase(**person_muir.dict())
+        person = PersonPublic(**person_muir.dict())
         access_token = auth_service.create_access_token_for_person(
             person=person,
             secret_key=str(SECRET_KEY),
@@ -624,7 +731,7 @@ class TestAuthTokens:
         exception: Type[BaseException],
     ) -> None:
         with pytest.raises(exception):
-            person = PersonBase(**person_muir.dict())
+            person = PersonPublic(**person_muir.dict())
             access_token = auth_service.create_access_token_for_person(
                 person=person,
                 secret_key=str(secret_key),
@@ -637,10 +744,10 @@ class TestAuthTokens:
     async def test_can_retrieve_username_from_token(
         self, app: FastAPI, client: AsyncClient, person_muir: UsualSuspect
     ) -> None:
-        person = PersonBase(**person_muir.dict())
+        person = PersonPublic(**person_muir.dict())
         token = auth_service.create_access_token_for_person(person=person, secret_key=str(SECRET_KEY))
-        username = auth_service.get_username_from_token(token=token, secret_key=str(SECRET_KEY))
-        assert username == person_muir.username
+        person_id = auth_service.get_person_id_from_token(token=token, secret_key=str(SECRET_KEY))
+        assert person_id == person_muir.id
 
     @pytest.mark.parametrize(
         "secret, wrong_token",
@@ -660,12 +767,12 @@ class TestAuthTokens:
         secret: Union[Secret, str],
         wrong_token: Optional[str],
     ) -> None:
-        person = PersonBase(**person_muir.dict())
+        person = PersonPublic(**person_muir.dict())
         token = auth_service.create_access_token_for_person(person=person, secret_key=str(SECRET_KEY))
         if wrong_token == "use correct token":
             wrong_token = token
         with pytest.raises(HTTPException):
-            auth_service.get_username_from_token(token=wrong_token, secret_key=str(secret))
+            auth_service.get_person_id_from_token(token=wrong_token, secret_key=str(secret))
 
 
 class TestPersonLogin:
@@ -791,14 +898,23 @@ class TestPersonMe:
             assert res.status_code == status.HTTP_200_OK
 
         assert res.json() == {
-            "active": IsTrueLike,
             "username": person_muir.username,
             "email_label": person_muir.email_label,
             "name_first": person_muir.name_first,
             "name_last": person_muir.name_last,
-            "id": IsPositiveInt,
+            "id": IsPositiveInt(),
             "created_at": IsDatetime(iso_string=True, enforce_tz=False),
-            "updated_at": IsDatetime(iso_string=True, enforce_tz=False)
+            "updated_at": IsDatetime(iso_string=True, enforce_tz=False),
+            'profile': {
+                'email_addresses': [
+                    {
+                        'email': person_muir.username,
+                        'email_label': person_muir.email_label,
+                        'email_primary': True,
+                        'verified': True
+                    }
+                ],
+            }
         }
 
     @pytest.mark.asyncio
